/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2012,2014 University of Zagreb
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Vedran Miletić <rivanvx@gmail.com>
 */

// Include a header file from your module to test.
#include "ns3/stop-restart-functionality.h"
#include "ns3/simulator.h"

// An essential include is test.h
#include "ns3/test.h"

// Do not put your test classes in namespace ns3.  You may find it useful
// to use the using directive to access the ns3 namespace directly
using namespace ns3;

class SimpleStartStopObject : public Object, public StopRestartFunctionality
{
public:
  static TypeId GetTypeId (void)
  {
    static TypeId tid = TypeId ("ns3::SimpleStartStopObject")
            .SetParent<Object> ()
            .AddConstructor<SimpleStartStopObject> ()
            .AddTraceSource ("StartTrace",
                             "Tracing change of object state to started",
                             MakeTraceSourceAccessor (&SimpleStartStopObject::m_restartTrace),
							 "void")
            .AddTraceSource ("StopTrace",
                             "Tracing change of object state to stop",
                             MakeTraceSourceAccessor (&SimpleStartStopObject::m_stopTrace),
							 "void")
            ;
    return tid;
  }

  SimpleStartStopObject ()
  {
  };
  ~SimpleStartStopObject ()
  {
  };
};

// ===========================================================================
// Test case to make sure that an Object Start and Stop functions work properly
// ===========================================================================
class StartStopFunctionalityTestCase : public TestCase
{
public:
  StartStopFunctionalityTestCase ();
  virtual ~StartStopFunctionalityTestCase ();

private:
  virtual void DoRun (void);
};

StartStopFunctionalityTestCase::StartStopFunctionalityTestCase () : TestCase ("Check Start and Stop functionality")
{
}

StartStopFunctionalityTestCase::~StartStopFunctionalityTestCase ()
{
}

void
StartStopFunctionalityTestCase::DoRun (void)
{
  Ptr<SimpleStartStopObject> objA = CreateObject<SimpleStartStopObject> ();

  //
  // Object is started by default
  //
  NS_TEST_ASSERT_MSG_EQ (objA->IsStarted (), true, "Object is unexpectedly not started");

  //
  // Stop an Object and test for it's state
  //
  objA->Stop ();
  NS_TEST_ASSERT_MSG_EQ (objA->IsStarted (), false, "Object is unexpectedly started");

  //
  // Start an Object and test for it's state
  //
  objA->Restart ();
  NS_TEST_ASSERT_MSG_EQ (objA->IsStarted (), true, "Object is unexpectedly not started");

  //
  // Stop an Object two times and test for it's state
  //
  objA->Stop ();
  objA->Stop ();
  NS_TEST_ASSERT_MSG_EQ (objA->IsStarted (), false, "Object is unexpectedly started");

  //
  // Start an Object two times and test for it's state
  //
  objA->Restart ();
  objA->Restart ();
  NS_TEST_ASSERT_MSG_EQ (objA->IsStarted (), true, "Object is unexpectedly not started");

  //
  // Simulator::Run and Simulator::Destroy don't affect state of an Object
  //
  Simulator::Run ();
  NS_TEST_ASSERT_MSG_EQ (objA->IsStarted (), true, "Object is unexpectedly not started");
  objA->Stop ();
  NS_TEST_ASSERT_MSG_EQ (objA->IsStarted (), false, "Object is unexpectedly started");
  objA->Restart ();
  NS_TEST_ASSERT_MSG_EQ (objA->IsStarted (), true, "Object is unexpectedly not started");
  Simulator::Destroy ();
  NS_TEST_ASSERT_MSG_EQ (objA->IsStarted (), true, "Object is unexpectedly not started");
  objA->Stop ();
  NS_TEST_ASSERT_MSG_EQ (objA->IsStarted (), false, "Object is unexpectedly started");
  objA->Restart ();
  NS_TEST_ASSERT_MSG_EQ (objA->IsStarted (), true, "Object is unexpectedly not started");
}

// ===========================================================================
// Test case to make sure that an Object Start and Stop tracking
// ===========================================================================
class StartStopTrackingTestCase : public TestCase
{
public:
  StartStopTrackingTestCase ();
  virtual ~StartStopTrackingTestCase ();

private:
  virtual void DoRun (void);
};

StartStopTrackingTestCase::StartStopTrackingTestCase () : TestCase ("Check Start and Stop tracking")
{
}

StartStopTrackingTestCase::~StartStopTrackingTestCase ()
{
}

void
StartStopTrackingTestCase::DoRun (void)
{

}

// The TestSuite class names the TestSuite, identifies what type of TestSuite,
// and enables the TestCases to be run.  Typically, only the constructor for
// this class must be defined
//
class StartStopFunctionalityTestSuite : public TestSuite
{
public:
  StartStopFunctionalityTestSuite ();
};

StartStopFunctionalityTestSuite::StartStopFunctionalityTestSuite ()
  : TestSuite ("start-stop-functionality", UNIT)
{
  // TestDuration for TestCase can be QUICK, EXTENSIVE or TAKES_FOREVER
  AddTestCase (new StartStopFunctionalityTestCase, TestCase::QUICK);
  AddTestCase (new StartStopTrackingTestCase, TestCase::QUICK);
}

// Do not forget to allocate an instance of this TestSuite
static StartStopFunctionalityTestSuite startStopFunctionalityTestSuite;

