/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2012,2014 University of Zagreb
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Vedran Miletić <rivanvx@gmail.com>
 */

#include "failure-repair-model.h"

#include <ns3/log.h>
#include <ns3/random-variable-stream.h>
#include <ns3/string.h>
#include <ns3/enum.h>
#include <ns3/uinteger.h>
#include <ns3/simulator.h>

NS_LOG_COMPONENT_DEFINE ("FailureRepairModel");

namespace ns3
{

NS_OBJECT_ENSURE_REGISTERED (FailureRepairModel);

TypeId
FailureRepairModel::GetTypeId (void)
{
  static TypeId tid = TypeId ("ns3::FailureRepairModel")
      .SetParent<Object> ()
      .AddConstructor<FailureRepairModel> ()
      .AddAttribute ("RepairedStateDuration",
                     "A RandomVariableStream used to pick the duration of the up state.",
                     StringValue ("ns3::ExponentialRandomVariable[Mean=1.0]"),
                     MakePointerAccessor (&FailureRepairModel::m_workingStateDuration),
                     MakePointerChecker<RandomVariableStream> ())
      .AddAttribute ("FailedStateDuration",
                     "A RandomVariableStream used to pick the duration of the failed state.",
                     StringValue ("ns3::ConstantRandomVariable[Constant=0.0]"),
                     MakePointerAccessor (&FailureRepairModel::m_failedStateDuration),
                     MakePointerChecker<RandomVariableStream> ())
      .AddAttribute ("TimeUnit",
                     "Time unit used by FailureRepairModel",
                     EnumValue (Time::S),
                     MakeEnumAccessor (&FailureRepairModel::m_timeUnit),
                     MakeEnumChecker (Time::Y, "Years",
                                      Time::D, "Days",
                                      Time::H, "Hours",
                                      Time::MIN, "Minutes",
                                      Time::S, "Seconds",
                                      Time::MS, "MilliSeconds",
                                      Time::US, "MicroSeconds",
                                      Time::NS, "NanoSeconds",
                                      Time::PS, "PicoSeconds",
                                      Time::FS, "FemtoSeconds"))
      .AddAttribute ("StateChangingTime",
                     "The amount of time the FailureRepairModel will change object state.",
                     TimeValue (Time (0)),
                     MakeTimeAccessor (&FailureRepairModel::m_stateChangingTime),
                     MakeTimeChecker ())
      .AddAttribute ("NumberOfStateChanges",
                     "The number of changes to object state (starts or stops) the FailureRepairModel will do.",
                     UintegerValue (0),
                     MakeUintegerAccessor (&FailureRepairModel::m_nStateChanges),
                     MakeUintegerChecker<uint32_t> ())
  ;
  return tid;
}

FailureRepairModel::FailureRepairModel ()
  : m_nStateChangesDone (0)
{
  NS_LOG_FUNCTION (this);
}

FailureRepairModel::~FailureRepairModel ()
{
  NS_LOG_FUNCTION (this);
}

void
FailureRepairModel::AssociateObject (Ptr<Object> obj, Callback<void> startCallback, Callback<void> stopCallback)
{
  AggregateObject (obj);
  m_startCallback = startCallback;
  m_stopCallback = stopCallback;
  Initialize ();
}

int64_t
FailureRepairModel::AssignStreams (int64_t stream)
{
  NS_LOG_FUNCTION (this << stream);

  m_workingStateDuration->SetStream (stream);
  m_failedStateDuration->SetStream (stream + 1);
  return 2;
}

uint32_t
FailureRepairModel::GetNStateChangesDone () const
{
  NS_LOG_FUNCTION (this);

  return m_nStateChangesDone;
}

void
FailureRepairModel::DoInitialize (void)
{
  NS_LOG_FUNCTION (this);

  NS_ASSERT_MSG (GetAggregateIterator ().HasNext (), "FailureRepairModel " << this << " has first to be attached to an object");
  NS_ASSERT_MSG (!m_startCallback.IsNull (), "StartCallback must be set");
  NS_ASSERT_MSG (!m_stopCallback.IsNull (), "StopCallback must be set");
  NS_ASSERT_MSG (m_nStateChanges != 0 || m_stateChangingTime != Time (0), "Number of state changes to do and state changing time can't both be 0.");

  NS_LOG_DEBUG ("State changing time " << m_stateChangingTime.As (m_timeUnit));
  NS_LOG_DEBUG ("Number of state changes " << m_nStateChanges);
  StartAttachedObject ();
}

void
FailureRepairModel::DoDispose (void)
{
  NS_LOG_FUNCTION (this);

  if (m_failedStateDuration != 0)
    {
      m_failedStateDuration->Dispose ();
      m_failedStateDuration = 0;
    }
  if (m_workingStateDuration != 0)
    {
      m_workingStateDuration->Dispose ();
      m_workingStateDuration = 0;
    }
  Object::DoDispose ();
}

void
FailureRepairModel::StartAttachedObject ()
{
  NS_LOG_FUNCTION (this);

  m_startCallback ();
  m_nStateChangesDone++;
  NS_LOG_INFO ("FailureRepairModel " << this << " has started attached object, state change number " << m_nStateChangesDone);
  
  if ((m_nStateChanges == 0 || m_nStateChangesDone < m_nStateChanges) && (m_stateChangingTime == Time (0) || Simulator::Now () < m_stateChangingTime))
    {
      Time startedStateInterval = Time::FromDouble (m_workingStateDuration->GetValue (), m_timeUnit);
      NS_LOG_INFO ("Stop of attached object scheduled " << startedStateInterval << " from now");
      m_stopObjectEvent = Simulator::Schedule (startedStateInterval, &FailureRepairModel::StopAttachedObject, this);
    }
  else
    {
      // This is needed to remove extra references to associated object so Dispose () gets called
      m_startCallback.Nullify ();
      m_stopCallback.Nullify ();
    }
}

void
FailureRepairModel::StopAttachedObject ()
{
  NS_LOG_FUNCTION (this);
  
  m_stopCallback ();
  m_nStateChangesDone++;
  NS_LOG_INFO ("FailureRepairModel " << this << " has stopped attached object, state change number " << m_nStateChangesDone);

  if ((m_nStateChanges == 0 || m_nStateChangesDone < m_nStateChanges) && (m_stateChangingTime == Time (0) || Simulator::Now () < m_stateChangingTime))
    {
      Time stoppedStateInterval = Time::FromDouble (m_failedStateDuration->GetValue (), m_timeUnit);
      NS_LOG_INFO ("Start of attached object scheduled " << stoppedStateInterval << " from now");
      m_startObjectEvent = Simulator::Schedule (stoppedStateInterval, &FailureRepairModel::StartAttachedObject, this);
    }
  else
    {
      // This is needed to remove extra references to associated object so Dispose () gets called
      m_startCallback.Nullify ();
      m_stopCallback.Nullify ();
    }
}

} // namespace ns3
