/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2012,2014 University of Zagreb
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Vedran Miletić <rivanvx@gmail.com>
 */

#ifndef FAILURE_REPAIR_MODEL_H
#define FAILURE_REPAIR_MODEL_H

#include <ns3/event-id.h>
#include <ns3/nstime.h>
#include <ns3/pointer.h>
#include <ns3/object.h>
#include <ns3/traced-callback.h>

#include <map>
#include <stdint.h>

namespace ns3
{

class RandomVariableStream;

/**
 * \ingroup startstop
 * \brief Started and stopped state changer
 *
 * Started and stopped state changer can be bound to an object and used to
 * automate the object state changes from started to stopped and back.
 * Started and stopped state durations are provided by RandomVariableStream
 * objects configured through attribute system. Started and stopped state
 * changer also requires user to set at least one among StateChangingTime and
 * NumberOfStateChanges attributes; default values for both are zero.
 */
class FailureRepairModel : public Object
{
public:
  static TypeId GetTypeId (void);
  FailureRepairModel ();
  virtual ~FailureRepairModel ();
  
  void AssociateObject (Ptr<Object> obj, Callback<void> startCallback, Callback<void> stopCallback);
  
  /**
   * Assign a fixed random variable stream number to the random variables
   * used by this model. Return the number of streams (possibly zero) that
   * have been assigned.
   *
   * \param stream first stream index to use
   * \return the number of stream indices assigned by this model
   */
  int64_t AssignStreams (int64_t stream);

  /**
   * FIXME
   */
  uint32_t GetNStateChangesDone () const;

protected:
  virtual void DoInitialize (void);
  /**
   * \brief Dispose StartedStoppedStateChanger
   * 
   * Note that this disposes attached StartedStoppedTimeTracker as well
   */
  virtual void DoDispose (void);
  
  /**
   * \brief Start attached object
   *
   * Execute Object::Start on attached Object and schedule the execution of
   * stop event
   */
  void StartAttachedObject ();

  /**
   * \brief Stop attached object
   *
   * Execute Object::Stop on attached Object and schedule the execution of
   * start event
   */
  void StopAttachedObject ();

private:
  Time::Unit m_timeUnit;
  
  uint32_t m_nStateChanges;
  uint32_t m_nStateChangesDone;
  Time m_stateChangingTime;
  
  Ptr<RandomVariableStream> m_workingStateDuration;
  Ptr<RandomVariableStream> m_failedStateDuration;

  EventId m_startObjectEvent;
  EventId m_stopObjectEvent;
  
  Callback<void> m_startCallback;
  Callback<void> m_stopCallback;
};

} // namespace ns3

#endif /* FAILURE_REPAIR_MODEL_H */

