/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2012,2014 University of Zagreb
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Vedran Miletić <rivanvx@gmail.com>
 */

#include "failure-repair-time-tracker.h"

#include <ns3/log.h>
#include <ns3/enum.h>
#include <ns3/random-variable-stream.h>
#include <ns3/simulator.h>

NS_LOG_COMPONENT_DEFINE ("FailureRepairTimeTracker");

namespace ns3
{

NS_OBJECT_ENSURE_REGISTERED (FailureRepairTimeTracker);

TypeId
FailureRepairTimeTracker::GetTypeId (void)
{
  static TypeId tid = TypeId ("ns3::FailureRepairTimeTracker")
      .SetParent<Object> ()
      .AddConstructor<FailureRepairTimeTracker> ()
  ;
  return tid;
}

FailureRepairTimeTracker::FailureRepairTimeTracker ()
  : m_lastEventType (UNKNOWN),
    m_totalTimeStarted (Time (0)),
    m_totalTimeStopped (Time (0)),
    m_object (0)
{
  NS_LOG_FUNCTION (this);
}

FailureRepairTimeTracker::~FailureRepairTimeTracker ()
{
  NS_LOG_FUNCTION (this);
}

void
FailureRepairTimeTracker::AssociateObject (Ptr<Object> obj, std::string repairTraceName, std::string failureTraceName)
{
  NS_LOG_FUNCTION (this << obj);

  NS_ASSERT (obj);
  NS_ASSERT_MSG (m_object == 0, "Cannot bind FailureRepairTimeTracker " << this << " to object " << obj << ", it is already bound to object " << m_object);
  obj->TraceConnectWithoutContext (failureTraceName, MakeCallback (&FailureRepairTimeTracker::TrackFailureEvent, this));
  obj->TraceConnectWithoutContext (repairTraceName, MakeCallback (&FailureRepairTimeTracker::TrackRepairEvent, this));
  m_object = obj;
  m_failureTraceName = failureTraceName;
  m_repairTraceName = repairTraceName;
  m_lastEventTime = Simulator::Now ();
}

void
FailureRepairTimeTracker::DiassociateObject (Ptr<Object> obj, std::string repairTraceName, std::string failureTraceName)
{
  NS_LOG_FUNCTION (this << obj);

  NS_ASSERT_MSG (obj, "Object can't be null");
  NS_ASSERT_MSG (obj == m_object, "Can't diassociate object which was not previously associated");
  NS_ASSERT_MSG (repairTraceName == m_repairTraceName, "Wrong failure trace name");
  NS_ASSERT_MSG (failureTraceName == m_failureTraceName, "Wrong repair trace name");
  m_object->TraceDisconnectWithoutContext (m_failureTraceName, MakeCallback (&FailureRepairTimeTracker::TrackFailureEvent, this));
  m_object->TraceDisconnectWithoutContext (m_repairTraceName, MakeCallback (&FailureRepairTimeTracker::TrackRepairEvent, this));
  m_object = 0;
}

Ptr<Object>
FailureRepairTimeTracker::GetAssociatedObject (void) const
{
  NS_LOG_FUNCTION (this);
  
  return m_object;
}

void
FailureRepairTimeTracker::TrackFailureEvent ()
{
  NS_LOG_FUNCTION (this);

  NS_LOG_LOGIC ("Tracking stop event");
  m_totalTimeStarted += (Simulator::Now () - m_lastEventTime);
  NS_LOG_INFO ("Total uptime increased by "
               << (Simulator::Now () - m_lastEventTime).GetSeconds () << "s to "
               << m_totalTimeStarted.GetSeconds () << "s");

  m_lastEventType = FAILURE;
  m_lastEventTime = Simulator::Now ();
}

void
FailureRepairTimeTracker::TrackRepairEvent ()
{
  NS_LOG_FUNCTION (this);
  
  NS_LOG_LOGIC ("Tracking start event");
  m_totalTimeStopped += (Simulator::Now () - m_lastEventTime);
  NS_LOG_INFO ("Total downtime increased by "
               << (Simulator::Now () - m_lastEventTime).GetSeconds () << "s to "
               << m_totalTimeStopped.GetSeconds () << "s");

  m_lastEventType = REPAIR;
  m_lastEventTime = Simulator::Now ();
}

Time
FailureRepairTimeTracker::GetTotalTimeStarted ()
{
  NS_LOG_FUNCTION (this);

  if (m_lastEventTime != Simulator::Now () && m_lastEventType == REPAIR)
    {
      NS_LOG_DEBUG ("Total time started is " << m_totalTimeStarted.GetSeconds () << "s");
      NS_LOG_DEBUG ("Time elapsed since last event is " << (Simulator::Now () - m_lastEventTime).GetSeconds () << "s");
    }

  return m_totalTimeStarted;
}

Time
FailureRepairTimeTracker::GetTotalTimeStopped ()
{
  NS_LOG_FUNCTION (this);
  
  if (m_lastEventTime != Simulator::Now () && m_lastEventType == FAILURE)
    {
      NS_LOG_DEBUG ("Total time started is " << m_totalTimeStarted.GetSeconds () << "s");
      NS_LOG_DEBUG ("Time elapsed since last event is " << (Simulator::Now () - m_lastEventTime).GetSeconds () << "s");
    }

  return m_totalTimeStopped;
}

double
FailureRepairTimeTracker::GetPercentageStarted ()
{
  NS_LOG_FUNCTION (this);

  double totalTimeStartedSeconds = GetTotalTimeStarted ().GetSeconds ();
  NS_LOG_INFO ("Total time started: " << totalTimeStartedSeconds << "s");
  Time totalTime = GetTotalTimeStarted () + GetTotalTimeStopped ();
  NS_LOG_INFO ("Total time: " << totalTime.GetSeconds () << "s");
  return totalTime.GetSeconds () == 0 ? 0
          : totalTimeStartedSeconds / totalTime.GetSeconds ();
}

double
FailureRepairTimeTracker::GetPercentageStopped ()
{
  NS_LOG_FUNCTION (this);

  double totalTimeStoppedSeconds = GetTotalTimeStopped ().GetSeconds ();
  NS_LOG_INFO ("Total time stopped: " << totalTimeStoppedSeconds << "s");
  Time totalTime = GetTotalTimeStarted () + GetTotalTimeStopped ();
  NS_LOG_INFO ("Total time: " << totalTime.GetSeconds () << "s");
  return totalTime.GetSeconds () == 0 ? 0
          : totalTimeStoppedSeconds / totalTime.GetSeconds ();
}

void
FailureRepairTimeTracker::DoDispose ()
{
  NS_LOG_FUNCTION (this);

  if (m_object != 0)
    {
      DiassociateObject (m_object, m_repairTraceName, m_failureTraceName);
    }

  Object::DoDispose ();
}

} // namespace ns3

