/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2012,2014 University of Zagreb
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Vedran Miletić <rivanvx@gmail.com>
 */

#ifndef FAILURE_REPAIR_TIME_TRACKER_H
#define FAILURE_REPAIR_TIME_TRACKER_H

#include <ns3/event-id.h>
#include <ns3/nstime.h>
#include <ns3/ptr.h>
#include <ns3/object.h>

#include <string>

namespace ns3
{

/**
 * \ingroup core
 * \defgroup startstop
 */
/**
 * \ingroup startstop
 * \brief Started and stopped time tracker
 *
 * Started and stopped time tracker can be bound to an object and used to track
 * the object state changes.
 */
class FailureRepairTimeTracker : public Object
{
public:
  static TypeId GetTypeId (void);
  FailureRepairTimeTracker ();
  virtual ~FailureRepairTimeTracker ();

  /**
   * This method attaches StartedStoppedTimeTracker to an Object.
   *
   * Note that StartedStoppedTimeTracker can be bound to only one object, i.e.
   * trying to attach it to two objects will produce an error.
   *
   * \param obj pointer to Object which StartedStoppedTimeTracker will track
   * \param traceName FIXME
   * \sa NotifyNewAggregate
   */
  void AssociateObject (Ptr<Object> obj, std::string failureTraceName, std::string repairTraceName);

  void DiassociateObject (Ptr<Object> obj, std::string failureTraceName, std::string repairTraceName);
  /**
   * TODO
   */
  Ptr<Object> GetAssociatedObject (void) const;
  
  /**
   * This method is used as a callback when attaching to an Object.
   *
   * \param from state of bound Object prior to state change
   * \param to state of bound Object after state change
   * \sa NotifyNewAggregate
   */
  void TrackFailureEvent ();
  
  /**
   * This method is used as a callback when attaching to an Object.
   *
   * \param from state of bound Object prior to state change
   * \param to state of bound Object after state change
   * \sa NotifyNewAggregate
   */
  void TrackRepairEvent ();

  /**
   * \return the amount of time the Object has been started
   */
  Time GetTotalTimeStarted ();
  
  /**
   * \return the amount of time the Object has been stopped
   */
  Time GetTotalTimeStopped ();

  /**
   * \return the percentage of time bound Object has been started
   */
  double GetPercentageStarted ();
  
  /**
   * \return the percentage of time bound Object has been stopped
   */
  double GetPercentageStopped ();

protected:
  virtual void DoDispose ();
  
  enum TrackedEventType
  {
    FAILURE,
    REPAIR,
    UNKNOWN
  };
  
private:
  TrackedEventType m_lastEventType;
  Time m_lastEventTime;
  Time m_totalTimeStarted;
  Time m_totalTimeStopped;

  Ptr<Object> m_object;
  std::string m_failureTraceName;
  std::string m_repairTraceName;
};

} // namespace ns3

#endif /* FAILURE_REPAIR_TIME_TRACKER_H */
